# Some functions linked to AR practical (T. Chateau) 
# It uses some lines from the sample plane_ar provindig by opencv
import numpy as np
import cv2
# Here we define the vertices and edges of the 3D wired model to apply
# Simple model of a house - cube with a triangular prism "roof"
ar_verts = np.float32([[0, 0, 0], [0, 1, 0], [1, 1, 0], [1, 0, 0],
                       [0, 0, 1], [0, 1, 1], [1, 1, 1], [1, 0, 1],
                       [0, 0.5, 2], [1, 0.5, 2]])
ar_edges = [(0, 1), (1, 2), (2, 3), (3, 0),
            (4, 5), (5, 6), (6, 7), (7, 4),
            (0, 4), (1, 5), (2, 6), (3, 7),
            (4, 8), (5, 8), (6, 9), (7, 9), (8, 9)]
            
# Load intrinsec matrix and distorsion corresponding to the camera used
# The camera can be calibrated using opencv
K = np.load('cam.npy')
dist_coef = np.load('dist.npy')

#Function to overlay 3D model
def draw_overlay(vis, x0,y0,x1,y1, xt0,yt0,xt1,yt1,xt2,yt2,xt3,yt3):
  # Parameters are the the upper left and lower right points of the triangle defining the reference patch
  # and the four points defining the current corners of the tracked patch
  #
  # 3D position of the 4 references corners
  quad_3d = np.float32([[x0, y0, 0], [x1, y0, 0], [x1, y1, 0], [x0, y1, 0]])
  # 2D position of these four corners in the current image
  quad = np.float32([[xt0, yt0], [xt1, yt1], [xt2, yt2], [xt3, yt3]])
  h, w = vis.shape[:2]
  # Here we call the solvePnP opencv function that returns both rotation and translation between the reference object and the current frame 
  _ret, rvec, tvec = cv2.solvePnP(quad_3d, quad, K, dist_coef)
  #print('rvec = ',rvec)
  #print('tvec = ',tvec)
  # projection of the vertices into the current frame using the computed pose
  verts = ar_verts * [(x1-x0), (y1-y0), -(x1-x0)*0.3] + (x0, y0, 0)
  verts = cv2.projectPoints(verts, rvec, tvec, K, dist_coef)[0].reshape(-1, 2)
  # Display the edges
  for i, j in ar_edges:
      (x0, y0), (x1, y1) = verts[i], verts[j]
      cv2.line(vis, (int(x0), int(y0)), (int(x1), int(y1)), (255, 255, 0), 2)
